// -------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture Reference
texture Reference <string uiname="ReferenceTexture";>;
float4x4 tReference: TEXTUREMATRIX <string uiname="Texture Transform";>;   //Texture Transform
sampler SampReference = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Reference);          //apply a texture to the sampler
    MipFilter = LINEAR;         //set the sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//texture Subtrahend
texture Subtrahend <string uiname="SubtrahendTexture";>;
float4x4 tSubtrahend: TEXTUREMATRIX <string uiname="Texture Transform";>;   //Texture Transform
sampler SampSubtrahend = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Subtrahend);          //apply a texture to the sampler
    MipFilter = LINEAR;         //set the sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};
//creates a threshold pin
float threshold <string uiname="Threshold";>  = 0.00;
//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tReference);

    return Out;
}

// -------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------

//bypass see the input
float4 bypass(vs2ps In): COLOR
{
    //texture lookup to access the pixels of the texture
    float4 col = tex2D(SampSubtrahend, In.TexCd);
    //do nothing
    return col;
}

//difference beetween texture reference and subtrahend (video in)
float4 difference(vs2ps In): COLOR
{
    //texture lookup to access the pixels of the texture
    float4 ColReference = tex2D(SampReference, In.TexCd);
    float4 ColSubtrahend = tex2D(SampSubtrahend, In.TexCd);
    //make the math difference
    float4 Difference = ColReference-ColSubtrahend;
    //make everything positiv
    //trick: small diff are less value then big diff
    Difference = Difference*Difference;
    //grayscale conversation - everything is converted into black&white
    float luma_dif = dot(float3(0.222, 0.707, 0.071), Difference.rgb);
    //trick: instad of if (luma_dif > threshold) then Difference alpha = 1
    //more faster than if (try to avoid ifs)
    Difference.a = luma_dif > threshold;
    //mask
    Difference.rgb = ColSubtrahend.rgb;
    return Difference;
}

// -------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------

technique bypass //name for the technique pin
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_4 bypass();
    }
}

technique difference //name for the technique pin
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_4 difference();
    }
}
