// this is an effect template. use it to start writing your own effects.

// -------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------


//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //set the sampler states LINEAR
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};
//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//kernel filter weights and the factor which is multiplicated
float factor <string uiname="factor";> = 1;

float weight1 <string uiname="weight1";> = 0;
float weight2 <string uiname="weight2";> = 0;
float weight3 <string uiname="weight3";> = 0;
float weight4 <string uiname="weight4";> = 0;
float weight5 <string uiname="weight5";> = 1;
float weight6 <string uiname="weight6";> = 0;
float weight7 <string uiname="weight7";> = 0;
float weight8 <string uiname="weight8";> = 0;
float weight9 <string uiname="weight9";> = 0;

//pin for pixeldiameter and the offset of kernel filter
float factorPixelOffset <string uiname="Offset";> = 1;
int TexWidth <string uiname="Texture Width";> = 512;
int TexHeight <string uiname="Texture Heihgt";> = 512;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
    float2 Offset : TEXCOORD1;
};

// -------------------------------------------------------------------------
// VERTEXSHADERS
// -------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);
    //factor 1 = pixeldiameter
    Out.Offset.x = 1*factorPixelOffset /float (TexWidth);
    Out.Offset.y = 1*factorPixelOffset /float (TexHeight);
    return Out;
}

// -------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------



//kernel filter
float4 PS3(vs2ps In): COLOR
{
    //offset.x and offset.y comes from vertex function
    //center
    float4 sampelCol = tex2D(Samp, In.TexCd)*weight5;
    //neighbours
    //
    //3x3 matrix start in left corner
    //upper row
    sampelCol += tex2D(Samp, float2(In.TexCd.x-In.Offset.x , In.TexCd.y+In.Offset.y)) *weight1;
    sampelCol += tex2D(Samp, float2(In.TexCd.x , In.TexCd.y+In.Offset.y)) *weight2;
    sampelCol += tex2D(Samp, float2(In.TexCd.x+In.Offset.x , In.TexCd.y+In.Offset.y)) *weight3;
    
    //left + right from kernel
    sampelCol += tex2D(Samp, float2(In.TexCd.x-In.Offset.x , In.TexCd.y)) *weight4;
    sampelCol += tex2D(Samp, float2(In.TexCd.x+In.Offset.x , In.TexCd.y)) *weight6;
    
    //lower row
    sampelCol += tex2D(Samp, float2(In.TexCd.x-In.Offset.x , In.TexCd.y-In.Offset.y)) *weight7;
    sampelCol += tex2D(Samp, float2(In.TexCd.x , In.TexCd.y-In.Offset.y)) *weight8;
    sampelCol += tex2D(Samp, float2(In.TexCd.x+In.Offset.x , In.TexCd.y-In.Offset.y)) *weight9;

    return sampelCol*factor;
}
// bypass
float4 PS0(vs2ps In): COLOR
{
    float4 col = tex2D(Samp, float2 (In.TexCd.x, In.TexCd.y));
    return col;
}

// -------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------

technique KernelFilter
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS3();
    }
}

technique Bypass
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS0();
    }
}

