// -------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = None;           //set the sampler states
    MinFilter = Point;
    MagFilter = Point;
    AddressU = wrap;
    AddressV = wrap;
};
//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//pin for pixeldiameter
float PixelFactor <string uiname="PixelDiameter Factor";>  = 1;
float width <string uiname="width";>  = 256;
float height <string uiname="height";>  = 256;
//float temp <string uiname="temp";>  = 0;

struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
    float2 Pixeldiameter : TEXCOORD1;
};
// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);
    Out.Pixeldiameter.x = 1*PixelFactor/width;
    Out.Pixeldiameter.y = 1*PixelFactor/height;
    return Out;
}

// -------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------

float4 GameOfLife(vs2ps In): COLOR
{
    /*
    # A dead cell with exactly three live neighbors becomes a live cell (birth).	
    # A live cell with two or three live neighbors stays alive (survival).          		
    # In all other cases, a cell dies or remains dead (overcrowding or loneliness).
    */
    //texture lookup to access the pixels of the texture
    float4 center = tex2D(Samp, float2 (In.TexCd.x, In.TexCd.y));
    //beighbours
    float4 up = tex2D(Samp, float2 (In.TexCd.x, In.TexCd.y+In.Pixeldiameter.y));
    float4 up_left = tex2D(Samp, float2 (In.TexCd.x-In.Pixeldiameter.x, In.TexCd.y+In.Pixeldiameter.y));
    float4 up_right = tex2D(Samp, float2 (In.TexCd.x+In.Pixeldiameter.x, In.TexCd.y+In.Pixeldiameter.y));
    float4 down = tex2D(Samp, float2 (In.TexCd.x, In.TexCd.y-In.Pixeldiameter.y));
    float4 down_left = tex2D(Samp, float2 (In.TexCd.x-In.Pixeldiameter.x, In.TexCd.y-In.Pixeldiameter.y));
    float4 down_right = tex2D(Samp, float2 (In.TexCd.x+In.Pixeldiameter.x, In.TexCd.y-In.Pixeldiameter.y));
    float4 left = tex2D(Samp, float2 (In.TexCd.x-In.Pixeldiameter.x, In.TexCd.y));
    float4 right = tex2D(Samp, float2 (In.TexCd.x+In.Pixeldiameter.x, In.TexCd.y));
    
    float4 alive = (up + up_left + up_right + down + down_left + down_right + left + right);

    if (center.r == 0 && alive.r == 3) {
      //A dead cell with exactly three live neighbors becomes a live cell (birth).	
      center.rgb =1;
    } else if (center.r == 1 && (alive.r == 3 || alive.r == 2)) {
      //A live cell with two or three live neighbors stays alive (survival).          		
      center.rgb =1;
    } else {
      //In all other cases, a cell dies or remains dead (overcrowding or loneliness).
      center.rgb =0;
    }

    return center;

}

float4 feedback(vs2ps In): COLOR
{
    //texture lookup to access the pixels of the texture
    float4 col = tex2D(Samp, float2(In.TexCd.x+In.Pixeldiameter.x,In.TexCd.y+In.Pixeldiameter.y));
    return col;
}


// -------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------

technique simpel_feedback //name for the technique pin
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_4 feedback();
    }
}

technique GameOfLife //name for the technique pin
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 GameOfLife();
    }
}

